<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Ethernet extends MX_Controller {

    /**
     * Index Page for this controller.
     *
     * Past the login page this loads the page for /network/ethernet.
     *
     * Note: This page has some caveats. On IP address change of the primary
     * interface we can't redirect to the new IP, because the CODB SET transaction
     * happens on POST and we never get the reply from the webserver back if the IP
     * changes during that. 
     *
     * So we do a few round abouts:
     *
     * - This GUI page does NOT do CODB SET transactions on SAVE. It stores them
     *   as JSON encoded array in a file that only AdmServ has access to.
     *
     * - Once that's one, we redirect to /network/ethernetDeploy, which has a
     *   meta-equif that reloads /network/ethernet on a working IP after so 
     *   and so many seconds.
     *
     * - /network/ethernetDeploy has an iFrame that loads /network/ethernetIframe
     *   and *that* actually loads the replay-file and does the CODB transactions.
     *
     * - Should /network/ethernetIframe be done before the hard coded redirect time
     *   has expired, it'll do a redirect itself.
     * 
     * All in all: This isn't perfect, but for the most part it works.
     *
     */

    public function index() {

        $CI =& get_instance();

        // We load the BlueOnyx helper library first of all, as we heavily depend on it:
        $this->load->helper('blueonyx');
        init_libraries();

        // Need to load 'BxPage' for page rendering:
        $this->load->library('BxPage');

        // Get $CI->BX_SESSION['sessionId'] and $CI->BX_SESSION['loginName'] from Cookie (if they are set):
        $CI->BX_SESSION['sessionId'] = $CI->input->cookie('sessionId');
        $CI->BX_SESSION['loginName'] = $CI->input->cookie('loginName');

        // Line up the ducks for CCE-Connection:
        include_once('ServerScriptHelper.php');
        $CI->serverScriptHelper = new ServerScriptHelper($CI->BX_SESSION['sessionId'], $CI->BX_SESSION['loginName']);
        $CI->cceClient = $CI->serverScriptHelper->getCceClient();
        $user = $CI->BX_SESSION['loginUser'];
        $i18n = new I18n("base-network", $CI->BX_SESSION['loginUser']['localePreference']);
        $system = $CI->getSystem();

        // Array to store CCEd transactions for later replay:
        $cce_replay = array();

        // Initialize Capabilities so that we can poll the access rights as well:
        $Capabilities = new Capabilities($CI->cceClient, $CI->BX_SESSION['loginName'], $CI->BX_SESSION['sessionId']);

        // -- Actual page logic start:

        // Not 'serverNetwork'? Bye, bye!
        if (!$Capabilities->getAllowed('serverNetwork')) {
            // Nice people say goodbye, or CCEd waits forever:
            $CI->cceClient->bye();
            $CI->serverScriptHelper->destructor();
            Log403Error("/gui/Forbidden403");
        }

        // Protect certain form fields read-only inside VPS's:
        if (is_file("/procx/user_beancounters")) {
            $fieldprot = "r";
        }
        else {
            $fieldprot = "rw";
        }

        // Are we running on AWS?
        if (is_file("/etc/is_aws")) {
            $is_aws = "1";
        }
        else {
            $is_aws = "0";
        }

        $redirect = "";

        //
        //--- Get CODB-Object of interest: 
        //

        $system = $CI->getSystem();

        //
        //--- Handle form validation:
        //

        // We start without any active errors:
        $errors = array();
        $extra_headers =array();
        $ci_errors = array();
        $my_errors = array();

        // Shove submitted input into $form_data after passing it through the XSS filter:
        $form_data = $CI->input->post(NULL, TRUE);

        // Form fields that are required to have input:
        $required_keys = array();
        // Set up rules for form validation. These validations happen before we submit to CCE and further checks based on the schemas are done:

        // Empty array for key => values we want to submit to CCE:
        $attributes = array();
        // Items we do NOT want to submit to CCE:
        $ignore_attributes = array("BlueOnyx_Info_Text");
        if (is_array($form_data)) {
            // Function GetFormAttributes() walks through the $form_data and returns us the $parameters we want to
            // submit to CCE. It intelligently handles checkboxes, which only have "on" set when they are ticked.
            // In that case it pulls the unticked status from the hidden checkboxes and addes them to $parameters.
            // It also transformes the value of the ticked checkboxes from "on" to "1". 
            //
            // Additionally it generates the form_validation rules for CodeIgniter.
            //
            // params: $i18n                i18n Object of the error messages
            // params: $form_data           array with form_data array from CI
            // params: $required_keys       array with keys that must have data in it. Needed for CodeIgniter's error checks
            // params: $ignore_attributes   array with items we want to ignore. Such as Labels.
            // return:                      array with keys and values ready to submit to CCE.
            $attributes = GetFormAttributes($i18n, $form_data, $required_keys, $ignore_attributes, $i18n);
        }
        //Setting up error messages:
        $CI->form_validation->set_message('required', $i18n->get("[[palette.val_is_required]]", false, array("field" => "\"%s\"")));        

        // Do we have validation related errors?
        if ($CI->form_validation->run() == FALSE) {

            if (validation_errors()) {
                // Set CI related errors:
                $ci_errors = array(validation_errors('<div class="alert dismissible alert_red"><img width="40" height="36" src="/.adm/images/icons/small/white/alarm_bell.png"><strong>', '</strong></div>'));
            }           
            else {
                // No errors. Pass empty array along:
                $ci_errors = array();
            }
        }

        //
        //--- Own error checks:
        //

        //
        //--- At this point all checks are done. If we have no errors, we can submit the data to CODB:
        //

        // Join the various error messages:
        $errors = array_merge($ci_errors, $my_errors);

        // If we have no errors and have POST data, we submit to CODB:
        if ((count($errors) == "0") && ($CI->input->post(NULL, TRUE))) {

            // We have no errors. We submit to CODB.

            $oids = $CI->cceClient->find("System");
            $product = new Product( $CI->cceClient );

            if ($product->isRaq()) {

                // Record CCEd transaction for later replay:
                $cce_replay[][$oids[0]] = array("hostname" => $attributes['hostNameField'], "domainname" => $attributes['domainNameField'], "dns" => $attributes['dnsAddressesField'], "gateway" => $attributes['gatewayField'] , "gateway_IPv6" => $attributes['gatewayField_IPv6'], "nw_update" => time());
            }
            else {
                $cce_replay[][$oids[0]] = array("hostname" => $attributes['hostNameField'], "domainname" => $attributes['domainNameField'], "dns" => $attributes['dnsAddressesField']);
            }

            //--> Redirect needs to be handled here.

            // handle all devices
            $devices = array('br0');
            $devices_new = array();
            if (isset($attributes['deviceList'])) {
                $devices = $CI->cceClient->scalar_to_array($attributes['deviceList']);
            }

            foreach ($devices as $key => $value) {
                if ($value != 'br0') {
                    unset($devices[$key]);
                }
            }

            // Ok, this is nuts. Somehow our '&br0&' got turned into '&br0;&' and I have no idea 
            // where or why this happened. So we have to walk through the $devides array and every
            // device in it needs to have any superfluxous ';' removed:
            foreach ($devices as $key => $value) {
                $devices_new[] = preg_replace('/;/', '', $value);
            }
            $devices = $devices_new;

            // special array for admin if errors
            $admin_if_errors = array();
            for ($i = 0; $i < count($devices); $i ++) {
                $var_name = "ipAddressField" . $devices[$i];
                $var_name_IPv6 = "IPv6_ipAddressField" . $devices[$i];
                $ip_field = $attributes[$var_name];

                if ($attributes['gatewayField_IPv6'] != '') {
                    $ip_field_IPv6 = $attributes[$var_name_IPv6];
                    $ip_orig_IPv6 = $attributes[$var_name_IPv6];
                }
                else {
                    $ip_field_IPv6 = '';
                    $ip_orig_IPv6 = '';
                }
                $var_name = "ipAddressOrig" . $devices[$i];
                $ip_orig = $attributes[$var_name];
                $var_name = "netMaskField" . $devices[$i];
                $nm_field = $attributes[$var_name];
                $var_name = "netMaskOrig" . $devices[$i];
                $nm_orig = $attributes[$var_name];
                $var_name = "bootProtoField" . $devices[$i];
                $boot_field = $attributes[$var_name];
               
                // setup or set disabled
                if (($ip_field == '') && ($ip_orig_IPv6 == '')) {

                }
                else {
                    $target_OID = $CI->cceClient->findx('Network', array('device' => "$devices[$i]"), array(), 'ascii', 'device');
                    if (isset($target_OID['0'])) {
                        $cce_replay[][$target_OID['0']] = array('ipaddr' => $ip_field, 'netmask' => $nm_field, 'ipaddr_IPv6' => $ip_field_IPv6, 'enabled' => 1, 'bootproto' => 'none', 'refresh' => time());
                    }
                }
            }

            $json_cce_replay = json_encode($cce_replay);
            $CI->load->helper('file');

            $cce_replay_file = '/usr/sausalito/license/json_cce_replay';

            // Delete old replay file:
            $ret = $CI->serverScriptHelper->shell("/bin/rm -f $cce_replay_file", $nfk, 'root', $CI->BX_SESSION['sessionId']);

            // Write the new license file out to disk:
            if (!write_file($cce_replay_file, $json_cce_replay)) {
                // Yadda
            }

            $url_param = '?';

            if ($attributes['IPv6_ipAddressFieldbr0'] != '') {
                $url_param .= 'ipv6=' . $attributes['IPv6_ipAddressFieldbr0'];
            }
            if ($attributes['ipAddressFieldbr0'] != '') {
                if ($url_param != '?') {
                    $url_param .= '&';
                }
                $url_param .= 'ipv4=' . $attributes['ipAddressFieldbr0'];
            }

            header("Location: /network/ethernetDeploy$url_param");
            exit;
        }

        //
        //-- Own page logic:
        //

        //
        //-- Generate page:
        //

        // Prepare Page:
        $factory = $CI->serverScriptHelper->getHtmlComponentFactory("base-network", "/network/ethernet");
        $BxPage = $factory->getPage();

        $framebreak = '<script language="JavaScript" type="text/javascript">' . "\n";
        $framebreak .= '<!--' . "\n";
        $framebreak .= 'function breakout_of_frame()' . "\n";
        $framebreak .= '{' . "\n";
        $framebreak .= '  if (top.location != location) {' . "\n";
        $framebreak .= '    top.location.href = document.location.href ;' . "\n";
        $framebreak .= '  }' . "\n";
        $framebreak .= '}' . "\n";
        $framebreak .= '-->' . "\n";
        $framebreak .= '</script>' . "\n";
        $BxPage->setExtraHeaders($framebreak);
        $BxPage->setExtraBodyTag('<body onload="breakout_of_frame()">');

        // Primary IP is changing. Show redirect 'error' message:
        if ($redirect != "") {
            $redir_msg[] = '<div class="alert dismissible alert_green"><img width="40" height="36" src="/.adm/images/icons/small/white/alarm_bell.png"><strong>' . $i18n->interpolateHtml('[[base-network.adminRedirect]]') . '</strong></div>';
            $errors = array_merge($redir_msg, $errors);
        }

        // Show OpenVZ message:
        if (is_file("/procx/user_beancounters")) {
            $vps_msg[] = '<div class="alert dismissible alert_green"><img width="40" height="36" src="/.adm/images/icons/small/white/alarm_bell.png"><strong>' . $i18n->interpolateHtml('[[base-network.openvz_vps]]') . '</strong></div>';
            $errors = array_merge($vps_msg, $errors);
        }

        // Get errorMsg from URL string.
        $get_form_data = $CI->input->get(NULL, TRUE);
        if (isset($get_form_data['errorMsg'])) {
            $errors[] = @json_decode(urldecode($get_form_data['errorMsg']));
        }

        $BxPage->setErrors($errors);
        $i18n = $factory->getI18n();

        $product = new Product($CI->cceClient);

        // Set Menu items:
        $BxPage->setVerticalMenu('base_serverconfig');
        $BxPage->setVerticalMenuChild('base_ethernet');
        $page_module = 'base_sysmanage';

        $default_page = 'primarySettings';
        if (($fieldprot == "rw") && ($is_aws == "0")) {
            // Show "Interface Aliasses" if not inside a VPS:
            //$pages = array($default_page, 'aliasSettings');
            $pages = array($default_page);
        }
        else {
            // Hide "Interface Aliasses" inside a VPS:
            $pages = array($default_page);
        }

        $block =& $factory->getPagedBlock("tcpIpSettings", $pages);

        $block->setToggle("#");
        $block->setSideTabs(FALSE);
        $block->setShowAllTabs("#");
        $block->setDefaultPage($default_page);

        if ($redirect != "") {
            $oldIP = $_SERVER['SERVER_ADDR'];
            $port = $_SERVER['SERVER_PORT'];
            $reconnect = $factory->getButton("http://$redirect:$port/network/ethernet", 'reconnect');
            $fallback = $factory->getButton("http://$oldIP:$port/network/ethernet", 'oldIPReconnect');

            $buttonRSContainer = $factory->getButtonContainer("tcpIpSettings", array($reconnect, $fallback));
            $block->addFormField(
                $buttonRSContainer,
                $factory->getLabel("tcpIpSettings"),
                $default_page
            );
        }

        //
        //--- TAB: primarySettings
        //

        // host and domain names
        $hostfield = $factory->getDomainName("hostNameField", $system["hostname"], $fieldprot);
        $domainfield = $factory->getDomainName("domainNameField", $system["domainname"], $fieldprot);

        $fqdn = $factory->getCompositeFormField(array($hostfield, $domainfield), '&nbsp;.&nbsp;');

        $block->addFormField(
            $fqdn,
            $factory->getLabel("enterFqdn"), 
            $default_page
        );

        $dns = $factory->getIpAddressList("dnsAddressesField", $system["dns"], $fieldprot);
        $dns->setOptional(true);
        $dns->setType('ipaddr_list_IPv4IPv6');
        $block->addFormField(
          $dns,
          $factory->getLabel("dnsAddressesField"),
          $default_page
        );

        if ($product->isRaq()) {
            if ($is_aws == "1") {
                if (!isset($system["gateway"])) {
                    // AWS and Gateway not defined. Make it editable:
                    $gwFprot = 'rw';
                }
                else {
                    if ($system["gateway"] == "") {
                        // AWS and Gateway not set. Make it editable:
                        $gwFprot = 'rw';
                    }
                    else {
                        // AWS, Gateway is set and not empty. Show it.
                        // But do not allow to edit it:
                        $gwFprot = 'r';
                    }
                }
            }
            else {
                // Not AWS. Allow edits if they are allowed for any of
                // the other network related fields:
                $gwFprot = $fieldprot;
            }
            $gw = $factory->getIpAddress("gatewayField", $system["gateway"], $gwFprot);
            $gw->setOptional(true);
            $block->addFormField($gw, $factory->getLabel("gatewayField"), $default_page);
        }

        if ($product->isRaq()) {
            if ($is_aws == "1") {
                if (!isset($system["gateway_IPv6"])) {
                    // AWS and Gateway not defined. Make it editable:
                    $gwFprot = 'rw';
                }
                else {
                    if ($system["gateway_IPv6"] == "") {
                        // AWS and Gateway not set. Make it editable:
                        $gwFprot = 'rw';
                    }
                    else {
                        // AWS, Gateway is set and not empty. Show it.
                        // But do not allow to edit it:
                        $gwFprot = 'r';
                    }
                }
            }
            else {
                // Not AWS. Allow edits if they are allowed for any of
                // the other network related fields:
                $gwFprot = $fieldprot;
            }
            $gw_IPv6 = $factory->getIpAddress("gatewayField_IPv6", $system["gateway_IPv6"], $gwFprot);
            $gw_IPv6->setOptional(true);
            $gw_IPv6->setType('ipaddrIPv6');
            $block->addFormField($gw_IPv6, $factory->getLabel("gatewayField_IPv6"), $default_page);
        }

        // real interfaces
        // ascii sorted, this may be a problem if there are more than 10 interfaces
        $interfaces = $CI->cceClient->findx('Network', array('real' => 1), array(), 'ascii', 'device');
        $devices = array();
        $deviceList = array();
        $devnames = array();
        $i18n = $factory->getI18n();
        $admin_if = '';
        for ($i = 0; $i < count($interfaces); $i++) {

            $is_admin_if = false;
            $iface = $CI->cceClient->get($interfaces[$i]);
            $device = $iface['device'];
            
            // save the devices and strings for javascript fun
            $deviceList[] = $device;
            $devices[] = "'$device'";    
            $devnames[] = "'" . $i18n->getJs("[[base-network.interface$device]]") . "'";

                // Devices:
                $dev[$device] = array (
                                'ipaddr' => $iface["ipaddr"],
                                'netmask' => $iface["netmask"],
                                'ipaddr_IPv6' => $iface["ipaddr_IPv6"],
                                'mac' => $iface["mac"],
                                'device' => $device,
                                'bootproto' => $iface["bootproto"],
                                'enabled' => $iface["enabled"]
                                );

        }

        if (isset($dev['br0'])) {
            $ipaddr = $dev['br0']['ipaddr'];
            $netmask = $dev['br0']['netmask'];
            $ipaddr_IPv6 = $dev['br0']['ipaddr_IPv6'];
            $device = $dev['br0']['device'];
            $mac = $dev['br0']['mac'];
            $enabled = $dev['br0']['enabled'];
            $bootproto = $dev['br0']['bootproto'];
            
            $ip_label = 'ipAddressField1';
            $nm_label = 'netMaskField1';
            $ip_label_IPv6 = 'IPv6_ipAddressField1';

            // Add divider:
            $block->addFormField(
                    $factory->addBXDivider("interface$device", ""),
                    $factory->getLabel("interface$device", false),
                    $default_page
                    );

            if ($is_aws == "0") {
                $devprot = "rw";
            }
            else {
                $devprot = "r";
            }

            $ip_field0 = $factory->getIpAddress("ipAddressField$device", $ipaddr, $devprot);
            $ip_field0->setInvalidMessage($i18n->getJs('ipAddressField_invalid'));
            $ip_field0->setCurrentLabel($i18n->getHtml('[[base-network.ipAddressField1]]', true, array(), array('name' => "[[base-network.help$device]]")));
            $ip_field0->setDescription($i18n->getWrapped('[[base-network.ipAddressField1_help]]', true, array(), array('name' => "[[base-network.help$device]]")));
            $ip_field0->setOptional(true);

            $block->addFormField(
                    $ip_field0,
                    $factory->getLabel($ip_label, true,
                                array(), array('name' => "[[base-network.help$device]]")),
                    $default_page
                );

            $netmask_field0 = $factory->getIpAddress("netMaskField$device", $netmask, $devprot);
            $netmask_field0->setInvalidMessage($i18n->getJs('netMaskField_invalid'));
            $netmask_field0->setCurrentLabel($i18n->getHtml('[[base-network.netMaskField1]]', true, array(), array('name' => "[[base-network.help$device]]")));
            $netmask_field0->setDescription($i18n->getWrapped('[[base-network.netMaskField1_help]]', true, array(), array('name' => "[[base-network.help$device]]")));

            // Netmask is not optional for the admin iface and for br0
            $netmask_field0->setOptional(true);
            
            $block->addFormField(
                    $netmask_field0,
                    $factory->getLabel($nm_label, true,
                                array(), array('name' => "[[base-network.help$device]]")),
                    $default_page
                );

            $IPv6_ip_field0 = $factory->getIpAddress("IPv6_ipAddressField$device", $ipaddr_IPv6, $devprot);
            $IPv6_ip_field0->setInvalidMessage($i18n->getJs('IPv6_ipAddressField_invalid'));
            $IPv6_ip_field0->setCurrentLabel($i18n->getHtml('[[base-network.IPv6_ipAddressField1]]', true, array(), array('name' => "[[base-network.help$device]]")));
            $IPv6_ip_field0->setDescription($i18n->getWrapped('[[base-network.IPv6_ipAddressField1_help]]', true, array(), array('name' => "[[base-network.help$device]]")));
            $IPv6_ip_field0->setOptional(true);
            $IPv6_ip_field0->setType('ipaddrIPv6');

            $block->addFormField(
                    $IPv6_ip_field0,
                    $factory->getLabel($ip_label_IPv6, true,
                                array(), array('name' => "[[base-network.help$device]]")),
                    $default_page
                );

            // MAC:
        //  $macaddress_field0 = $factory->getMacAddress("macAddressField$device", $mac, "r");
        //  $macaddress_field0->setCurrentLabel($i18n->getHtml('[[base-network.macAddressField]]', true));
        //  $macaddress_field0->setDescription($i18n->getWrapped('[[base-network.macAddressField_help]]', true));

        //    $block->addFormField(
        //            $macaddress_field0,
        //            $factory->getLabel("macAddressField"),
        //            $default_page
        //        );

            // retain orginal information
            $block->addFormField(
                    $factory->getBoolean("hasAliases$device", 0, ''));

            $block->addFormField(
                    $factory->getIpAddress("ipAddressOrig$device", $ipaddr, ""),
                    '',
                    $default_page
                    );
            $block->addFormField(
                    $factory->getIpAddress("netMaskOrig$device", $netmask, ""),
                    "",
                    $default_page
                    );
            $block->addFormField(
                    $factory->getTextField("bootProtoField$device", $bootproto, ""),
                    "",
                    $default_page
                    );
            $block->addFormField(
                    $factory->getBoolean("enabled$device", $enabled, ""),
                    "",
                    $default_page
                    );

        }

        // add a hidden field indicating which interface is the admin interface
        $block->addFormField($factory->getTextField('adminIf', 'br0', ''));
        $block->addFormField($factory->getTextField('deviceList', $CI->cceClient->array_to_scalar($deviceList), ''));

        //
        //--- TAB: aliasSettings
        //

        if ((!is_file("/procx/user_beancounters")) && (!is_file("/etc/is_aws"))) {

            // Add-Button:
            $addAlias = "/network/aliasModify";
            $addbutton = $factory->getAddButton($addAlias, '[[base-network.addAliasButton]]', "DEMO-OVERRIDE");
            $buttonContainer = $factory->getButtonContainer("aliasSettings", $addbutton);
            $block->addFormField(
                $buttonContainer,
                $factory->getLabel("aliasSettings"),
                'aliasSettings'
            );


            // add scrollist of aliases
            $alias_list = $factory->getScrollList("aliasSettings", array('aliasName', 'aliasIpaddr', 'aliasNetmask', 'aliasActions'), array()); 
            $alias_list->setAlignments(array("left", "left", "center", "center"));
            $alias_list->setDefaultSortedIndex('0');
            $alias_list->setSortOrder('ascending');
            $alias_list->setSortDisabled(array('3'));
            $alias_list->setPaginateDisabled(FALSE);
            $alias_list->setSearchDisabled(FALSE);
            $alias_list->setSelectorDisabled(FALSE);
            $alias_list->enableAutoWidth(FALSE);
            $alias_list->setInfoDisabled(FALSE);
            $alias_list->setColumnWidths(array("320", "178", "120", "120")); // Max: 739px
        
            $sort_map = array('device', 'ipaddr', 'netmask');
            $networks = $CI->cceClient->findx(
                          'Network', array('real' => 0), array(),
                          'ascii', $sort_map[$alias_list->getSortedIndex()]);

            for($i=0; $i < count($networks); $i++) {
                // must be an alias
                $alias = $CI->cceClient->get($networks[$i]);
                if ((isset($device_info[0])) && (isset($device_info[1]))) {
                    $device_info = preg_split('/:/', $alias['device']);
                    $alias_name = $i18n->interpolateHtml('[[base-network.alias' . $device_info[0] . ']]', array('num' => $device_info[1]));
                    
                    $modButt = $factory->getModifyButton("/network/aliasModify?ACTION=M&_oid=$networks[$i]");
                    $modButt->setImageOnly(TRUE);
                    $delButt = $factory->getRemoveButton("/network/aliasModify?ACTION=D&_oid=$networks[$i]");
                    $delButt->setImageOnly(TRUE);
                
                    $alias_list->addEntry(
                                  array(
                                    $alias_name,
                                    $alias['ipaddr'],
                                    $alias['netmask'],
                                    $factory->getCompositeFormField(
                                                    array(
                                                      $modButt,
                                                      $delButt
                                                      )
                                                    )
                                    ));
                }
              }

            // Push out the Scrollist with the aliasSettings:
            $block->addFormField(
                $factory->getRawHTML("aliasSettings", $alias_list->toHtml()),
                $factory->getLabel("aliasSettings"),
                'aliasSettings'
            );
        }

        //
        //--- Add the buttons
        //

        // Only add the save button if looking at primary settings AND we're not inside a VPS:
        if ($fieldprot == "rw") {
            $block->addButton($factory->getSaveButton($BxPage->getSubmitAction()));
            $block->addButton($factory->getCancelButton("/network/ethernet"));
        }


        //
        //-- Static Routes GUI? Disabled for now:
        //

        //$routeButton = $factory->getButton("/network/routes", "routes", "DEMO-OVERRIDE");
        //$buttonRouteContainer = $factory->getButtonContainer(" ", array($routeButton));
        //$page_body[] = $buttonRouteContainer->toHtml();

        $page_body[] = $block->toHtml();

        // Out with the page:
        $BxPage->render($page_module, $page_body);

    }       
}

/*
Copyright (c) 2014-2017 Michael Stauber, SOLARSPEED.NET
Copyright (c) 2014-2017 Team BlueOnyx, BLUEONYX.IT
All Rights Reserved.

1. Redistributions of source code must retain the above copyright 
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright 
   notice, this list of conditions and the following disclaimer in 
   the documentation and/or other materials provided with the 
   distribution.

3. Neither the name of the copyright holder nor the names of its 
   contributors may be used to endorse or promote products derived 
   from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.

You acknowledge that this software is not designed or intended for 
use in the design, construction, operation or maintenance of any 
nuclear facility.

*/
?>